@@ -7,16 +7,16 @@
 #ifndef SOLVESPACE_H
 #define SOLVESPACE_H
 
-#include <ctype.h>
-#include <limits.h>
-#include <math.h>
-#include <setjmp.h>
-#include <stdarg.h>
-#include <stddef.h>
-#include <stdint.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
+#include <cctype>
+#include <climits>
+#include <cmath>
+#include <csetjmp>
+#include <cstdarg>
+#include <cstddef>
+#include <cstdint>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <algorithm>
 #include <chrono>
 #include <functional>
@@ -71,19 +71,19 @@ typedef struct _cairo_surface cairo_surface_t;
     } while(0)
 #endif
 
-#ifndef isnan
-#   define isnan(x) (((x) != (x)) || (x > 1e11) || (x < -1e11))
-#endif
+#define dbp SolveSpace::Platform::DebugPrint
+#define DBPTRI(tri) \
+    dbp("tri: (%.3f %.3f %.3f) (%.3f %.3f %.3f) (%.3f %.3f %.3f)", \
+        CO((tri).a), CO((tri).b), CO((tri).c))
 
 namespace SolveSpace {
 
 using std::min;
 using std::max;
 using std::swap;
+using std::fabs;
 
-#if defined(__GNUC__)
-__attribute__((noreturn))
-#endif
+[[noreturn]]
 void AssertFailure(const char *file, unsigned line, const char *function,
                    const char *condition, const char *message);
 
@@ -92,6 +92,10 @@ __attribute__((__format__ (__printf__, 1, 2)))
 #endif
 std::string ssprintf(const char *fmt, ...);
 
+inline bool IsReasonable(double x) {
+    return std::isnan(x) || x > 1e11 || x < -1e11;
+}
+
 inline int WRAP(int v, int n) {
     // Clamp it to the range [0, n)
     while(v >= n) v -= n;
@@ -111,54 +115,30 @@ inline double WRAP_SYMMETRIC(double v, double n) {
     return v;
 }
 
-// Why is this faster than the library function?
-inline double ffabs(double v) { return (v > 0) ? v : (-v); }
-
 #define CO(v) (v).x, (v).y, (v).z
 
-#define ANGLE_COS_EPS   (1e-6)
-#define LENGTH_EPS      (1e-6)
-#define VERY_POSITIVE   (1e10)
-#define VERY_NEGATIVE   (-1e10)
+static constexpr double ANGLE_COS_EPS =  1e-6;
+static constexpr double LENGTH_EPS    =  1e-6;
+static constexpr double VERY_POSITIVE =  1e10;
+static constexpr double VERY_NEGATIVE = -1e10;
 
 inline double Random(double vmax) {
     return (vmax*rand()) / RAND_MAX;
 }
 
+#include "platform/platform.h"
+#include "platform/gui.h"
+#include "resource.h"
+
+using Platform::AllocTemporary;
+using Platform::FreeAllTemporary;
+
 class Expr;
 class ExprVector;
 class ExprQuaternion;
 class RgbaColor;
 enum class Command : uint32_t;
 
-//================
-// From the platform-specific code.
-
-#include "platform/platform.h"
-#include "platform/gui.h"
-
-const size_t MAX_RECENT = 8;
-
-#define AUTOSAVE_EXT "slvs~"
-
-void dbp(const char *str, ...);
-#define DBPTRI(tri) \
-    dbp("tri: (%.3f %.3f %.3f) (%.3f %.3f %.3f) (%.3f %.3f %.3f)", \
-        CO((tri).a), CO((tri).b), CO((tri).c))
-
-std::vector<std::string> InitPlatform(int argc, char **argv);
-
-void *AllocTemporary(size_t n);
-void FreeAllTemporary();
-void *MemAlloc(size_t n);
-void MemFree(void *p);
-void vl(); // debug function to validate heaps
-
-// End of platform-specific functions
-//================
-
-#include "resource.h"
-
 enum class Unit : uint32_t {
     MM = 0,
     INCHES,
@@ -675,7 +655,9 @@ public:
     static void MenuFile(Command id);
     void Autosave();
     void RemoveAutosave();
-    static const size_t MAX_RECENT = 8;
+    static constexpr size_t MAX_RECENT = 8;
+    static constexpr const char *SKETCH_EXT = "slvs";
+    static constexpr const char *BACKUP_EXT = "slvs~";
     std::vector<Platform::Path> recentFiles;
     bool Load(const Platform::Path &filename);
     bool GetFilenameAndSave(bool saveAs);
@@ -810,8 +792,8 @@ public:
     // where it puts zero-initialized global data in the binary (~30M of zeroes)
     // in release builds.
     SolveSpaceUI()
-        : pTW(new TextWindow({})), TW(*pTW),
-          pSys(new System({})), sys(*pSys) {}
+        : pTW(new TextWindow()), TW(*pTW),
+          pSys(new System()), sys(*pSys) {}
 
     ~SolveSpaceUI() {
         delete pTW;
